# Copyright 2022-2023 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API Core Objects."""

from .job import BaseJob, ExtractJob, JobSettings, ListJobs, SummarizeJob
from .source import (
    AssemblyAISource,
    AudioSource,
    BaseSource,
    DeepgramSource,
    GenericSource,
    InMemorySource,
    RevSource,
    SignedURLSource,
    VTTSource,
    WordcabTranscriptSource,
)
from .stats import Stats
from .summary import BaseSummary, ListSummaries, StructuredSummary
from .transcript import BaseTranscript, ListTranscripts, TranscriptUtterance


__all__ = [
    "AssemblyAISource",
    "AudioSource",
    "BaseJob",
    "BaseSource",
    "BaseSummary",
    "BaseTranscript",
    "DeepgramSource",
    "ExtractJob",
    "GenericSource",
    "InMemorySource",
    "JobSettings",
    "ListJobs",
    "ListSummaries",
    "ListTranscripts",
    "RevSource",
    "SignedURLSource",
    "Stats",
    "StructuredSummary",
    "SummarizeJob",
    "TranscriptUtterance",
    "VTTSource",
    "WordcabTranscriptSource",
]



# Copyright 2022-2023 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API Client."""

import logging
from typing import Dict, List, Optional, Union, no_type_check

import requests  # type: ignore

from .config import (
    CONTEXT_ELEMENTS,
    EXTRACT_PIPELINES,
    LIST_JOBS_ORDER_BY,
    REQUEST_TIMEOUT,
    SOURCE_LANG,
    SOURCE_OBJECT_MAPPING,
    SUMMARY_LENGTHS_RANGE,
    SUMMARY_PIPELINES,
    SUMMARY_TYPES,
    TARGET_LANG,
)
from .core_objects import (
    BaseSource,
    BaseSummary,
    BaseTranscript,
    ExtractJob,
    InMemorySource,
    JobSettings,
    ListJobs,
    ListSummaries,
    ListTranscripts,
    Stats,
    StructuredSummary,
    SummarizeJob,
    TranscriptUtterance,
    WordcabTranscriptSource,
)
from .login import get_token
from .utils import (
    _check_context_elements,
    _check_extract_pipelines,
    _check_source_lang,
    _check_summary_length,
    _check_summary_pipelines,
    _check_target_lang,
    _format_context_elements,
    _format_lengths,
    _format_pipelines,
    _format_tags,
)


logger = logging.getLogger(__name__)


class Client:
    """Wordcab API Client."""

    def __init__(self, api_key: Optional[str] = None):
        """Initialize the client."""
        self.api_key = api_key if api_key else get_token()
        if not self.api_key:
            raise ValueError(
                """
            API Key not found. You must set the WORDCAB_API_KEY environment variable. Use `wordcab login` to login
            to the Wordcab CLI and set the environment variable.
            """
            )
        self.timeout = REQUEST_TIMEOUT

    def __enter__(self) -> "Client":
        """Enter the client context."""
        return self

    def __exit__(
        self,
        exception_type: Optional[Union[ValueError, TypeError, AssertionError]],
        exception_value: Optional[Exception],
        traceback: Optional[Exception],
    ) -> None:
        """Exit the client context."""
        pass

    @no_type_check
    def request(
        self,
        method: str,
        **kwargs: Union[bool, int, str, Dict[str, str], List[int], List[str]],
    ) -> Union[
        BaseSource,
        BaseSummary,
        BaseTranscript,
        ExtractJob,
        ListJobs,
        ListSummaries,
        ListTranscripts,
        Stats,
        SummarizeJob,
        Union[ExtractJob, SummarizeJob],
    ]:
        """Make a request to the Wordcab API."""
        if not method:
            raise ValueError("You must specify a method.")
        return getattr(self, method)(**kwargs)

    def get_stats(
        self,
        min_created: Optional[str] = None,
        max_created: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ) -> Stats:
        """Get the stats of the account."""
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Accept": "application/json",
        }
        params: Dict[str, str] = {}
        if min_created:
            params["min_created"] = min_created
        if max_created:
            params["max_created"] = max_created
        if tags:
            params["tags"] = _format_tags(tags)

        r = requests.get(
            "https://wordcab.com/api/v1/me",
            headers=headers,
            params=params,
            timeout=self.timeout,
        )

        if r.status_code == 200:
            return Stats(**r.json())
        else:
            raise ValueError(r.text)

    def start_extract(  # noqa: C901
        self,
        source_object: Union[BaseSource, InMemorySource, WordcabTranscriptSource],
        display_name: str,
        ephemeral_data: Optional[bool] = False,
        only_api: Optional[bool] = True,
        pipelines: Union[str, List[str]] = [  # noqa: B006
            "questions_answers",
            "topic_segments",
            "emotions",
            "speaker_talk_ratios",
        ],
        split_long_utterances: Optional[bool] = False,
        tags: Optional[Union[str, List[str]]] = None,
    ) -> ExtractJob:
        """Start an Extraction job."""
        if _check_extract_pipelines(pipelines) is False:
            raise ValueError(
                f"""
                You must specify a valid list of pipelines.
                Available pipelines are: {", ".join(EXTRACT_PIPELINES[:-1])} and {EXTRACT_PIPELINES[-1]}.
            """
            )
        if (
            isinstance(
                source_object, (BaseSource, InMemorySource, WordcabTranscriptSource)
            )
            is False
        ):
            raise ValueError(
                """
                You must specify a valid source object for the extraction job.
                See https://docs.wordcab.com/docs/accepted-sources for more information.
            """
            )

        source = source_object.source
        if source not in SOURCE_OBJECT_MAPPING.keys():
            raise ValueError(
                f"Invalid source: {source}. Source must be one of {SOURCE_OBJECT_MAPPING.keys()}"
            )
        if (
            source_object.__class__.__name__ != SOURCE_OBJECT_MAPPING[source]
            and source_object.__class__.__name__ != "InMemorySource"
        ):
            raise ValueError(
                f"""
                Invalid source object: {source_object}. Source object must be of type {SOURCE_OBJECT_MAPPING[source]},
                but is of type {type(source_object)}.
            """
            )

        if hasattr(source_object, "payload"):
            payload = source_object.payload
        else:
            payload = source_object.prepare_payload()

        if hasattr(source_object, "headers"):
            headers = source_object.headers
        else:
            headers = source_object.prepare_headers()
        headers["Authorization"] = f"Bearer {self.api_key}"

        pipelines = _format_pipelines(pipelines)
        params: Dict[str, Union[str, None]] = {
            "source": source,
            "display_name": display_name,
            "ephemeral_data": str(ephemeral_data).lower(),
            "only_api": str(only_api).lower(),
            "pipeline": pipelines,
            "split_long_utterances": str(split_long_utterances).lower(),
        }
        if tags:
            params["tags"] = _format_tags(tags)

        if source == "wordcab_transcript" and hasattr(source_object, "transcript_id"):
            params["transcript_id"] = source_object.transcript_id
        if source == "signed_url" and hasattr(source_object, "signed_url"):
            params["signed_url"] = source_object.signed_url

        if source == "audio" or source == "vtt":
            r = requests.post(
                "https://wordcab.com/api/v1/extract",
                headers=headers,
                params=params,
                files=payload,
                timeout=self.timeout,
            )
        else:
            r = requests.post(
                "https://wordcab.com/api/v1/extract",
                headers=headers,
                params=params,
                data=payload,
                timeout=self.timeout,
            )

        if r.status_code == 201:
            logger.info("Extract job started.")
            return ExtractJob(
                display_name=display_name,
                job_name=r.json()["job_name"],
                source=source,
                settings=JobSettings(
                    ephemeral_data=ephemeral_data,
                    only_api=only_api,
                    pipeline=pipelines,
                    split_long_utterances=split_long_utterances,
                ),
            )
        else:
            raise ValueError(r.text)

    def start_summary(  # noqa: C901
        self,
        source_object: Union[BaseSource, InMemorySource, WordcabTranscriptSource],
        display_name: str,
        summary_type: str,
        context: Optional[Union[str, List[str]]] = None,
        ephemeral_data: Optional[bool] = False,
        only_api: Optional[bool] = True,
        pipelines: Union[str, List[str]] = ["transcribe", "summarize"],  # noqa: B006
        source_lang: Optional[str] = None,
        split_long_utterances: Optional[bool] = False,
        summary_lens: Optional[Union[int, List[int]]] = None,
        target_lang: Optional[str] = None,
        tags: Optional[Union[str, List[str]]] = None,
    ) -> SummarizeJob:
        """Start a Summary job."""
        if summary_type not in SUMMARY_TYPES:
            raise ValueError(
                f"Invalid summary type. Available types are: {', '.join(SUMMARY_TYPES)}"
            )

        if summary_type == "reason_conclusion":
            raise ValueError(
                """
                The summary type 'reason_conclusion' has been removed. You can use `brief` instead.
            """
            )
        else:
            if summary_lens is None:
                logger.warning(
                    "You have not specified a summary length. Defaulting to 3."
                )
                summary_lens = 3
            if _check_summary_length(summary_lens) is False:
                raise ValueError(
                    f"""
                    You must specify a valid summary length. Summary length must be an integer or a list of integers.
                    The integer values must be between {SUMMARY_LENGTHS_RANGE[0]} and {SUMMARY_LENGTHS_RANGE[1]}.
                """
                )

        if _check_summary_pipelines(pipelines) is False:
            raise ValueError(
                f"""
                You must specify a valid list of pipelines.
                Available pipelines are: {", ".join(SUMMARY_PIPELINES[:-1])} and {SUMMARY_PIPELINES[-1]}.
            """
            )

        if (
            isinstance(
                source_object, (BaseSource, InMemorySource, WordcabTranscriptSource)
            )
            is False
        ):
            raise ValueError(
                """
                You must specify a valid source object to summarize.
                See https://docs.wordcab.com/docs/accepted-sources for more information.
            """
            )

        if _check_context_elements(context) is False:
            raise ValueError(
                f"""
                You must specify valid context elements. Context elements must be a string or a list of strings.
                Here are the available context elements: {", ".join(CONTEXT_ELEMENTS[:-1])} and {CONTEXT_ELEMENTS[-1]}.
            """
            )

        if source_lang is None:
            source_lang = "en"

        if target_lang is None:
            target_lang = source_lang

        if _check_source_lang(source_lang) is False:
            raise ValueError(
                f"""
                You must specify a valid source language.
                Available languages are: {", ".join(SOURCE_LANG[:-1])} or {SOURCE_LANG[-1]}.
            """
            )
        elif _check_target_lang(target_lang) is False:
            raise ValueError(
                f"""
                You must specify a valid target language.
                Available languages are: {", ".join(TARGET_LANG[:-1])} or {TARGET_LANG[-1]}.
            """
            )
        elif source_lang != "en" or target_lang != "en":
            logger.warning(
                """
                Languages outside `en` are currently in beta and may not be as accurate as the English model.
                We are working to improve the accuracy of the non-English models.
                If you have any feedback, don't hesitate to get in touch with us at info@wordcab.com.
            """
            )

        source = source_object.source
        if source not in SOURCE_OBJECT_MAPPING.keys():
            raise ValueError(
                f"Invalid source: {source}. Source must be one of {SOURCE_OBJECT_MAPPING.keys()}"
            )
        if (
            source_object.__class__.__name__ != SOURCE_OBJECT_MAPPING[source]
            and source_object.__class__.__name__ != "InMemorySource"
        ):
            raise ValueError(
                f"""
                Invalid source object: {source_object}. Source object must be of type {SOURCE_OBJECT_MAPPING[source]},
                but is of type {type(source_object)}.
            """
            )

        if hasattr(source_object, "payload"):
            payload = source_object.payload
        else:
            payload = source_object.prepare_payload()

        if hasattr(source_object, "headers"):
            headers = source_object.headers
        else:
            headers = source_object.prepare_headers()
        headers["Authorization"] = f"Bearer {self.api_key}"

        pipelines = _format_pipelines(pipelines)
        params: Dict[str, Union[str, None]] = {
            "source": source,
            "display_name": display_name,
            "ephemeral_data": str(ephemeral_data).lower(),
            "only_api": str(only_api).lower(),
            "pipeline": pipelines,
            "source_lang": source_lang,
            "target_lang": target_lang,
            "split_long_utterances": str(split_long_utterances).lower(),
            "summary_type": summary_type,
        }
        if context:
            params["context"] = _format_context_elements(context)
        if summary_lens:
            params["summary_lens"] = _format_lengths(summary_lens)
        if tags:
            params["tags"] = _format_tags(tags)

        if source == "wordcab_transcript" and hasattr(source_object, "transcript_id"):
            params["transcript_id"] = source_object.transcript_id
        if source == "signed_url" and hasattr(source_object, "signed_url"):
            params["signed_url"] = source_object.signed_url

        if source == "audio":
            r = requests.post(
                "https://wordcab.com/api/v1/summarize",
                headers=headers,
                params=params,
                files=payload,
                timeout=self.timeout,
            )
        else:
            r = requests.post(
                "https://wordcab.com/api/v1/summarize",
                headers=headers,
                params=params,
                data=payload,
                timeout=self.timeout,
            )

        if r.status_code == 201:
            logger.info("Summary job started.")
            return SummarizeJob(
                display_name=display_name,
                job_name=r.json()["job_name"],
                source=source,
                settings=JobSettings(
                    ephemeral_data=ephemeral_data,
                    pipeline=pipelines,
                    split_long_utterances=split_long_utterances,
                    only_api=only_api,
                ),
            )
        else:
            raise ValueError(r.text)

    def list_jobs(
        self,
        page_size: Optional[int] = 100,
        page_number: Optional[int] = None,
        order_by: Optional[str] = "-time_started",
    ) -> ListJobs:
        """List all jobs."""
        if order_by not in LIST_JOBS_ORDER_BY:
            raise ValueError(
                f"""
                Invalid `order_by` parameter. Must be one of {LIST_JOBS_ORDER_BY}.
                You can use - to indicate descending order.
            """
            )

        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Accept": "application/json",
        }
        params = {"page_size": page_size, "order_by": order_by}

        if page_number is not None:
            params["page"] = page_number

        r = requests.get(
            "https://wordcab.com/api/v1/jobs",
            headers=headers,
            params=params,
            timeout=self.timeout,
        )

        if r.status_code == 200:
            data = r.json()
            list_jobs: List[Union[ExtractJob, SummarizeJob]] = []
            for job in data["results"]:
                if "summary_details" in job:
                    list_jobs.append(SummarizeJob(**job))
                else:
                    list_jobs.append(ExtractJob(**job))
            return ListJobs(
                page_count=int(data["page_count"]),
                next_page=data.get("next"),
                results=list_jobs,
            )
        else:
            raise ValueError(r.text)

    def retrieve_job(self, job_name: str) -> Union[ExtractJob, SummarizeJob]:
        """Retrieve a job."""
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Accept": "application/json",
        }

        r = requests.get(
            f"https://wordcab.com/api/v1/jobs/{job_name}",
            headers=headers,
            timeout=self.timeout,
        )

        if r.status_code == 200:
            data = r.json()
            if "summary_details" in data:
                return SummarizeJob(**data)
            else:
                return ExtractJob(**data)
        else:
            raise ValueError(r.text)

    @no_type_check
    def delete_job(self, job_name: str, warning: bool = True) -> Dict[str, str]:
        """Delete a job."""
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Accept": "application/json",
        }

        r = requests.delete(
            f"https://wordcab.com/api/v1/jobs/{job_name}",
            headers=headers,
            timeout=self.timeout,
        )

        if r.status_code == 200:
            if warning:
                logger.warning(f"Job {job_name} deleted.")
            return r.json()
        else:
            raise ValueError(r.text)

    def list_transcripts(
        self, page_size: Optional[int] = 100, page_number: Optional[int] = None
    ) -> ListTranscripts:
        """List all transcripts."""
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Accept": "application/json",
        }
        params = {"page_size": page_size}

        if page_number is not None:
            params["page"] = page_number

        r = requests.get(
            "https://wordcab.com/api/v1/transcripts",
            headers=headers,
            params=params,
            timeout=self.timeout,
        )

        if r.status_code == 200:
            data = r.json()
            return ListTranscripts(
                page_count=int(data["page_count"]),
                next_page=data.get("next"),
                results=[
                    BaseTranscript(**transcript) for transcript in data["results"]
                ],
            )
        else:
            raise ValueError(r.text)

    def retrieve_transcript(self, transcript_id: str) -> BaseTranscript:
        """Retrieve a transcript."""
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Accept": "application/json",
        }

        r = requests.get(
            f"https://wordcab.com/api/v1/transcripts/{transcript_id}",
            headers=headers,
            timeout=self.timeout,
        )

        if r.status_code == 200:
            data = r.json()
            utterances = data.pop("transcript")
            transcript = BaseTranscript(**data)
            for utterance in utterances:
                transcript.transcript.append(TranscriptUtterance(**utterance))
            return transcript
        else:
            raise ValueError(r.text)

    def change_speaker_labels(
        self, transcript_id: str, speaker_map: Dict[str, str]
    ) -> BaseTranscript:
        """Change the speaker labels of a transcript."""
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Accept": "application/json",
        }

        r = requests.patch(
            f"https://wordcab.com/api/v1/transcripts/{transcript_id}",
            headers=headers,
            json={"speaker_map": speaker_map},
            timeout=self.timeout,
        )

        if r.status_code == 200:
            logger.info("Speaker labels changed.")
            return BaseTranscript(**r.json())
        else:
            raise ValueError(r.text)

    def list_summaries(
        self, page_size: Optional[int] = 100, page_number: Optional[int] = None
    ) -> ListSummaries:
        """List all summaries."""
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Accept": "application/json",
        }
        params = {"page_size": page_size}

        if page_number is not None:
            params["page"] = page_number

        r = requests.get(
            "https://wordcab.com/api/v1/summaries",
            headers=headers,
            params=params,
            timeout=self.timeout,
        )

        if r.status_code == 200:
            data = r.json()
            return ListSummaries(
                page_count=int(data["page_count"]),
                next_page=data.get("next"),
                results=[BaseSummary(**summary) for summary in data["results"]],
            )
        else:
            raise ValueError(r.text)

    def retrieve_summary(self, summary_id: str) -> BaseSummary:
        """Retrieve a summary."""
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Accept": "application/json",
        }

        r = requests.get(
            f"https://wordcab.com/api/v1/summaries/{summary_id}",
            headers=headers,
            timeout=self.timeout,
        )

        if r.status_code == 200:
            data = r.json()
            structured_summaries = data.pop("summary")
            summary = BaseSummary(**data)
            summaries: Dict[
                str,
                Dict[str, List[StructuredSummary]],
            ] = {}
            for key, value in structured_summaries.items():
                summaries[key] = {
                    "structured_summary": [
                        StructuredSummary(**items)
                        for items in value["structured_summary"]
                    ]
                }
            summary.summary = summaries
            return summary
        else:
            raise ValueError(r.text)



# Copyright 2023 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab core objects utils functions."""

import textwrap
from typing import Any, Dict


def _get_context_items(
    context: Dict[str, Any],
) -> str:
    """Get the context items."""
    context_items = ""

    if "issue" in context:
        context_items += f"Issue: {context['issue']}\n"

    if "purpose" in context:
        context_items += f"Purpose: {context['purpose']}\n"

    if "next_steps" in context:
        context_items += f"Next steps: {context['next_steps']}\n"

    if "discussion_points" in context:
        context_items += f"Discussion points: {context['discussion_points']}\n"

    if "keywords" in context:
        context_items += f"Keywords: {context['keywords']}\n"

    return context_items


def _get_assembly_utterances(assembly_json: Dict[str, Any]) -> Any:
    """Get the AssemblyAI utterances."""
    utterances = assembly_json.pop("utterances", None)

    if utterances is None:
        raise ValueError(
            "No utterances key found. Verify the AssemblyAI json file you are using."
        )

    return utterances


def _get_deepgram_utterances(deepgram_json: Dict[str, Any]) -> Any:
    """Get the Deepgram utterances."""
    results = deepgram_json.pop("results", None)

    if results is None:
        raise ValueError(
            "No results key found. Verify the Deepgram json file you are using."
        )

    utterances = results.pop("utterances", None)

    if utterances is None:
        raise ValueError(
            "No utterances key found. Verify the Deepgram json file you are using."
        )

    return utterances


def _get_rev_monologues(rev_json: Dict[str, Any]) -> Any:
    """Get the Rev.ai monologues."""
    monologues = rev_json.pop("monologues", None)

    if monologues is None:
        raise ValueError(
            "No monologues key found. Verify the Rev.ai json file you are using."
        )

    return monologues


def _textwrap(text_to_wrap: str, width: int = 80) -> str:
    """
    Return a formatted string with the text wrapped to the specified width using textwrap.

    Parameters
    ----------
    text_to_wrap : str
        The text to wrap.
    width : int
        The width to wrap the text to, by default 80.

    Returns
    -------
    str
        The formatted string with the text wrapped to the specified width.
    """
    return "\n".join(textwrap.wrap(text_to_wrap, width=width))



# Copyright 2022-2023 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API Stats object."""

import logging
from dataclasses import dataclass, field
from typing import List, Optional

import validators  # type: ignore

from ..config import AVAILABLE_PLAN


logger = logging.getLogger(__name__)


@dataclass
class Stats:
    """Stats object for the Wordcab API."""

    account_email: str
    plan: str
    monthly_request_limit: str
    request_count: int
    minutes_summarized: int
    transcripts_summarized: int
    metered_charge: str
    min_created: str
    max_created: str
    tags: Optional[List[str]] = field(default=None)

    def __post_init__(self) -> None:
        """Post-init method."""
        if self.plan not in AVAILABLE_PLAN:
            raise ValueError(f"Plan must be one of {AVAILABLE_PLAN}, not {self.plan}")

        if not validators.email(self.account_email):
            raise ValueError(
                f"Please provide a valid email address, not {self.account_email}"
            )

        # Check if metered charge is in the right format $X or $X.XX
        if not self.metered_charge.startswith("$"):
            raise ValueError(
                f"Metered charge must start with $, not {self.metered_charge}"
            )
        if not self.metered_charge[1:].replace(".", "", 1).isdigit():
            raise ValueError(
                f"Metered charge must be a number, not {self.metered_charge}"
            )

        if self.min_created > self.max_created:
            raise ValueError(
                f"min_created must be before max_created, not {self.min_created} and {self.max_created}"
            )



# Copyright 2022-2023 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab Python SDK."""

from .api import (
    change_speaker_labels,
    delete_job,
    get_stats,
    list_jobs,
    list_summaries,
    list_transcripts,
    request,
    retrieve_job,
    retrieve_summary,
    retrieve_transcript,
    start_extract,
    start_summary,
)
from .client import Client
from .login import login


__all__ = [
    "Client",
    "change_speaker_labels",
    "delete_job",
    "get_stats",
    "list_jobs",
    "list_summaries",
    "list_transcripts",
    "login",
    "request",
    "retrieve_job",
    "retrieve_summary",
    "retrieve_transcript",
    "start_extract",
    "start_summary",
]

__version__ = "0.6.5"



# Copyright 2022-2023 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API Summary object."""

import logging
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional, Union

from ..config import SUMMARY_TYPES
from .utils import _get_context_items, _textwrap


logger = logging.getLogger(__name__)


@dataclass
class StructuredSummary:
    """Structured summary object."""

    summary: Union[str, Dict[str, str]]
    context: Optional[
        Dict[str, Union[str, List[str], Dict[str, Union[str, List[str]]]]]
    ] = field(default=None)
    summary_html: Optional[Union[str, Dict[str, str]]] = field(default=None)
    end: Optional[str] = field(default=None)
    end_index: Optional[int] = field(default=None)
    start: Optional[str] = field(default=None)
    start_index: Optional[int] = field(default=None)
    timestamp_end: Optional[int] = field(default=None)
    timestamp_start: Optional[int] = field(default=None)
    transcript_segment: Optional[List[Dict[str, Union[str, int]]]] = field(default=None)

    def __repr__(self) -> str:
        """Return a string representation of the object without the None values."""
        return f"{self.__class__.__name__}({', '.join(f'{k}={v!r}' for k, v in self.__dict__.items() if v is not None)})"


@dataclass
class BaseSummary:
    """Summary object."""

    job_status: str
    summary_id: str
    display_name: Optional[str] = field(default=None)
    job_name: Optional[str] = field(default=None)
    process_time: Optional[str] = field(default=None)
    speaker_map: Optional[Dict[str, str]] = field(default=None)
    source: Optional[str] = field(default=None)
    source_lang: Optional[str] = field(default=None)
    summary_type: Optional[str] = field(default=None)
    summary: Optional[Dict[str, Any]] = field(default=None)
    target_lang: Optional[str] = field(default=None)
    transcript_id: Optional[str] = field(default=None)
    time_started: Optional[str] = field(default=None)
    time_completed: Optional[str] = field(default=None)

    def __post_init__(self) -> None:
        """Post init."""
        if self.summary_type:
            if self.summary_type not in SUMMARY_TYPES:
                raise ValueError(
                    f"Summary type must be one of {SUMMARY_TYPES}, not {self.summary_type}"
                )

        if self.time_started and self.time_completed:
            if self.time_started == self.time_completed:
                raise ValueError("time_started and time_completed must be different")

    def get_summaries(self) -> Dict[str, List[Union[str, List[str]]]]:
        """
        Get the summaries as a dictionary with the summary length as key and the summaries as values.

        Returns
        -------
        Dict[str, List[Union[str, List[str]]]]
            The summaries as a dictionnary with the summary length as key and the summaries as values.
            If the summary type is brief, the summaries are returned as a list of list of str,
            otherwise they are returned as a list of str.
        """
        if self.summary is None:
            return {}

        summaries: Dict[str, Any] = {}

        for summary_len in self.summary:
            summaries_list = [
                s.summary for s in self.summary[summary_len]["structured_summary"]
            ]

            if self.summary_type == "brief":
                summaries_list = [
                    [s["title"], s["brief_summary"]] for s in summaries_list
                ]

            summaries[summary_len] = summaries_list

        return summaries

    def get_formatted_summaries(
        self, add_context: Optional[bool] = False
    ) -> Dict[str, str]:
        """Format the summaries in an human readable format.

        Return the summaries as a dictionary in an human readable format with the summary length as key
        and the summaries as values.

        Parameters
        ----------
        add_context : bool, optional
            If True, add the context items to the summary, by default False.

        Returns
        -------
        Dict[str, str]
            The summaries as a dictionary with the summary length as key and the summaries as values formatted
            in an human readable format.
        """
        if self.summary is None:
            return {}

        summaries: Dict[str, Any] = {}

        if self.summary is None:
            raise ValueError("No summary available.")

        if self.summary_type is None:
            raise ValueError("No summary type found.")

        for summary_len in self.summary:
            summaries[summary_len] = _format_summary(
                self.summary[summary_len]["structured_summary"],
                self.summary_type,
                summary_len,
                add_context,
            )

        return summaries


@dataclass
class ListSummaries:
    """List summaries object."""

    page_count: int
    next_page: str
    results: List[BaseSummary]


def _format_summary(
    structured_summaries: List[StructuredSummary],
    summary_type: str,
    summary_len: str,
    add_context: Optional[bool] = False,
) -> str:
    """
    Format the summary in an human readable format.

    Parameters
    ----------
    structured_summaries : list
        The structured summaries list.
    summary_type : str
        The summary type.
    summary_len : str
        The summary length.
    add_context : bool, optional
        If True, add the context items to the summary. By default False.

    Returns
    -------
    str
        The summary formatted in an human readable format.
    """
    total_summary = len(structured_summaries)

    txt = f"{summary_type} - length: {summary_len}\n\n"
    for i in range(total_summary):
        txt += f"[{i + 1}/{total_summary}]\n"

        summary = structured_summaries[i].summary

        if isinstance(summary, dict):
            txt += f"Title: {summary['title']}\nSummary: {summary['brief_summary']}\n\n"

        if isinstance(summary, str):
            txt += f"{_textwrap(summary)}\n\n"

        if add_context:
            context_items = structured_summaries[i].context

            if context_items is not None:
                txt += f"{_get_context_items(context_items)}\n\n"

    return txt



# Copyright 2022-2023 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API Source object."""

import json
import logging
import urllib.parse
from dataclasses import dataclass, field
from pathlib import Path
from typing import Dict, List, Optional, Union, no_type_check

import requests  # type: ignore
import validators  # type: ignore

from ..config import AVAILABLE_AUDIO_FORMATS, AVAILABLE_GENERIC_FORMATS, REQUEST_TIMEOUT
from .utils import (
    _get_assembly_utterances,
    _get_deepgram_utterances,
    _get_rev_monologues,
)


logger = logging.getLogger(__name__)


@dataclass
class BaseSource:
    """Base class for AudioSource and GenericSource objects. It is not meant to be used directly.

    Parameters
    ----------
    filepath : Optional[Union[str, Path]], optional
        Path to the local file, by default None.
    url : Optional[str], optional
        URL to the remote file, by default None.
    url_headers : Optional[Dict[str, str]], optional
        Headers to retrieve the file from the URL, by default None.
        Useful if the file requires authentication to be retrieved.

    Raises
    ------
    ValueError
        If neither `filepath` nor `url` are provided.
    ValueError
        If both `filepath` and `url` are provided.
    TypeError
        If `filepath` is not a string or a Path object.
    FileNotFoundError
        If `filepath` does not exist or is not accessible.

    Attributes
    ----------
    source : str
        The source type.
    source_type : str
        The source type.
    _stem : str
        The stem of the file.
    _suffix : str
        The suffix of the file.

    Returns
    -------
    BaseSource
        The source object.
    """

    filepath: Optional[Union[str, Path]] = field(default=None, repr=False)
    url: Optional[str] = field(default=None, repr=False)
    url_headers: Optional[Dict[str, str]] = field(default=None, repr=False)
    source: str = field(init=False)
    source_type: str = field(init=False)
    _stem: str = field(init=False, repr=False)
    _suffix: str = field(init=False, repr=False)

    def __post_init__(self) -> None:
        """Post-init method."""
        self.source = self.__class__.__name__
        if not self.filepath and not self.url:
            raise ValueError(
                "Please provide either a local or a remote source, respectively `filepath` or `url`."
            )
        if self.filepath and self.url:
            raise ValueError(
                "Please provide either a local or a remote source, not both `filepath` and `url`."
            )

        if self.filepath:
            if not isinstance(self.filepath, Path) and not isinstance(
                self.filepath, str
            ):
                raise TypeError(
                    f"The path must be a string or a Path object, not {type(self.filepath)}"
                )

            if isinstance(self.filepath, str):
                self.filepath = Path(self.filepath)

            if not self.filepath.exists():
                raise FileNotFoundError(
                    f"File {self.filepath} does not exist or is not accessible."
                )

            self._stem = self.filepath.stem
            self._suffix = self.filepath.suffix
            self.source_type = "local"

        if self.url:
            if self._check_if_url_is_valid():
                filename = urllib.parse.unquote(self.url.split("/")[-1])
                self._stem = ".".join(filename.split(".")[:-1])
                self._suffix = f".{filename.split('.')[-1].split('?')[0]}"
                self.source_type = "remote"

    @no_type_check
    def _load_file_from_path(self) -> bytes:
        """Load file from local path."""
        with open(self.filepath, "rb") as f:
            return f.read()

    def _load_file_from_url(self) -> requests.Response.content:
        """Load file from URL."""
        if self.url_headers:
            file = requests.get(
                self.url, headers=self.url_headers, timeout=REQUEST_TIMEOUT
            )
        else:
            file = requests.get(self.url, timeout=REQUEST_TIMEOUT)

        return file.content

    def _check_if_url_is_valid(self) -> bool:
        """Check if URL is valid."""
        if not validators.url(self.url):
            raise ValueError(f"Please provide a valid URL. {self.url} is not valid.")

        return True

    def prepare_payload(self) -> Union[str, bytes, Dict[str, bytes]]:
        """Prepare payload."""
        raise NotImplementedError("Payload preparation is not implemented yet.")

    def prepare_headers(self) -> Dict[str, str]:
        """Prepare headers."""
        raise NotImplementedError("Headers preparation is not implemented yet.")


@dataclass
class InMemorySource:
    """In-memory source object.

    The in-memory source object is a special case of the generic source object.
    It is used to pass a pre-processed transcript to the API.

    Parameters
    ----------
    obj : Union[Dict[str, List[str]], List[str]]
        The in-memory object. It can be a list of strings or a dict with a `transcript` key
        and a list of strings as value.

    Raises
    ------
    ValueError
        If the in-memory object does not have a `transcript` key.
    TypeError
        If the in-memory object does not have a list as value for the `transcript` key.
    TypeError
        If the in-memory object is not a list or a dict.

    Examples
    --------
    >>> from wordcab.core_objects import InMemorySource

    >>> transcript = {"transcript": ["SPEAKER A: Hello.", "SPEAKER B: Hi."]}
    >>> in_memory_source = InMemorySource(obj=transcript)
    >>> in_memory_source
    InMemorySource(...)
    >>> in_memory_source.obj

    Returns
    -------
    InMemorySource
        The in-memory source object.
    """

    obj: Optional[Union[Dict[str, List[str]], List[str]]] = field(
        default=None, repr=False
    )
    source: str = field(init=False)
    source_type: str = field(init=False)

    def __post_init__(self) -> None:
        """Post-init method."""
        self.source = "generic"
        self.source_type = "in_memory"
        if isinstance(self.obj, dict):
            if "transcript" not in self.obj:
                raise ValueError(
                    "Please provide a valid in-memory object. It must have a `transcript` key."
                )
            elif not isinstance(self.obj["transcript"], list):
                raise TypeError(
                    "Please provide a valid in-memory object. The `transcript` key must be a list."
                )
        elif isinstance(self.obj, list):
            self.obj = {"transcript": self.obj}
        else:
            raise TypeError(
                "Please provide a valid in-memory object. It must be a list or a dict."
            )

    def prepare_payload(self) -> str:
        """Prepare payload for API request."""
        self.payload = json.dumps(self.obj)
        return self.payload

    def prepare_headers(self) -> Dict[str, str]:
        """Prepare headers for API request."""
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }
        return self.headers


@dataclass
class GenericSource(BaseSource):
    """Generic source object.

    The GenericSource object is required to create a job that uses a generic file as input,
    such as `.txt` or `.json` file.

    Parameters
    ----------
    filepath : Union[str, Path]
        The path to the local file.
    url : str
        The URL to the remote file.

    Raises
    ------
    ValueError
        If the file format is not supported.
    ValueError
        If both `filepath` and `url` are provided.
    TypeError
        If the path is not a string or a Path object.
    FileNotFoundError
        If the file does not exist or is not accessible.

    Examples
    --------
    >>> from wordcab.core_objects import GenericSource

    >>> generic_source = GenericSource(filepath="path/to/generic/file.txt")  # doctest: +SKIP
    >>> generic_source  # doctest: +SKIP
    GenericSource(...)
    >>> generic_source.file_object  # doctest: +SKIP
    b'Hello, world!'
    >>> generic_source.source_type  # doctest: +SKIP
    'local'
    >>> generic_source._suffix  # doctest: +SKIP
    '.txt'
    >>> generic_source._stem  # doctest: +SKIP
    'file'

    Returns
    -------
    GenericSource
        The generic source object.
    """

    file_object: bytes = field(init=False, repr=False)

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.source = "generic"

        if self._suffix not in AVAILABLE_GENERIC_FORMATS:
            raise ValueError(
                f"Please provide a valid file format. {self._suffix} is not valid."
            )
        if self.source_type == "local":
            self.file_object = self._load_file_from_path()
        elif self.source_type == "remote":
            self.file_object = self._load_file_from_url()

    def prepare_payload(self) -> str:
        """Prepare payload for API request."""
        if self._suffix == ".json":
            self.payload = json.dumps({"transcript": json.loads(self.file_object)})
        elif self._suffix == ".txt":
            self.payload = json.dumps(
                {"transcript": self.file_object.decode("utf-8").splitlines()}
            )
        return self.payload

    def prepare_headers(self) -> Dict[str, str]:
        """Prepare headers for API request."""
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }
        return self.headers


@dataclass
class AudioSource(BaseSource):
    """
    The AudioSource object is required to create a job that uses an audio file as input.

    Parameters
    ----------
    filepath : Union[str, Path]
        The path to the local file.
    url : str
        The URL to the remote file.

    Raises
    ------
    ValueError
        If the file format is not supported.
    ValueError
        If both `filepath` and `url` are provided.
    TypeError
        If the path is not a string or a Path object.
    FileNotFoundError
        If the file does not exist or is not accessible.

    Examples
    --------
    >>> from wordcab.core_objects import AudioSource

    >>> audio_source = AudioSource(filepath="path/to/audio/file.mp3")  # doctest: +SKIP
    >>> audio_source  # doctest: +SKIP
    AudioSource(...)

    Returns
    -------
    AudioSource
        The audio source object.
    """

    file_object: bytes = field(init=False, repr=False)

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.source = "audio"
        if self._suffix not in AVAILABLE_AUDIO_FORMATS:
            raise ValueError(
                f"Please provide a valid file format. {self._suffix} is not valid."
            )
        if self.source_type == "local":
            self.file_object = self._load_file_from_path()
        elif self.source_type == "remote":
            self.file_object = self._load_file_from_url()

    def prepare_payload(self) -> Dict[str, bytes]:
        """Prepare payload for API request."""
        self.payload = {"audio_file": self.file_object}
        return self.payload

    @no_type_check
    def prepare_headers(self) -> dict:
        """Prepare headers for API request."""
        self.headers = {}
        return self.headers


@dataclass
class WordcabTranscriptSource:
    """
    Wordcab transcript source object using a Wordcab transcript ID.

    Parameters
    ----------
    transcript_id : str
        The Wordcab transcript ID to use as input.

    Raises
    ------
    ValueError
        If the `transcript_id` is not provided.

    Examples
    --------
    >>> from wordcab.core_objects import WordcabTranscriptSource

    >>> wordcab_transcript_source = WordcabTranscriptSource(transcript_id="transcript_12345")
    >>> wordcab_transcript_source
    WordcabTranscriptSource(transcript_id=transcript_12345)

    Returns
    -------
    WordcabTranscriptSource
        The Wordcab transcript source object.
    """

    transcript_id: Optional[str] = field(default=None)
    source: str = field(init=False)

    def __post_init__(self) -> None:
        """Post-init method."""
        if self.transcript_id is None:
            raise ValueError(
                "Please provide a `transcript_id` to initialize a WordcabTranscriptSource object."
            )
        self.source = "wordcab_transcript"

    def __repr__(self) -> str:
        """Representation method."""
        return f"WordcabTranscriptSource(transcript_id={self.transcript_id})"

    def prepare_payload(self) -> None:
        """Prepare payload for API request."""
        return None

    def prepare_headers(self) -> Dict[str, str]:
        """Prepare headers for API request."""
        self.headers = {"Accept": "application/json"}
        return self.headers


@dataclass
class SignedURLSource(BaseSource):
    """Signed URL source object."""

    signed_url: Optional[str] = field(init=False)

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.signed_url = self.url
        self.source = "signed_url"
        raise NotImplementedError("Signed URL source is not implemented yet.")


@dataclass
class AssemblyAISource(BaseSource):
    """
    AssemblyAI source object using a local or remote AssemblyAI JSON file.

    Parameters
    ----------
    filepath : Union[str, Path]
        The path to the local file.
    url : str
        The URL to the remote file.

    Raises
    ------
    ValueError
        If the file format is not valid.

    Examples
    --------
    >>> from wordcab.core_objects import AssemblyAISource

    >>> assemblyai_source = AssemblyAISource(filepath="path/to/assemblyai/file.json")  # doctest: +SKIP
    >>> assemblyai_source  # doctest: +SKIP
    AssemblyAISource(...)
    >>> assemblyai_source.source  # doctest: +SKIP
    'assembly_ai'

    Returns
    -------
    AssemblyAISource
        The AssemblyAI source object.
    """

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.source = "assembly_ai"

        if self._suffix != ".json":
            raise ValueError(
                f"Please provide a valid AssemblyAI file format. {self._suffix} is not valid, it should be .json."
            )

        if self.source_type == "local":
            self.file_object = self._load_file_from_path()
        elif self.source_type == "remote":
            self.file_object = self._load_file_from_url()

    def prepare_payload(self) -> str:
        """Prepare payload for API request."""
        self.payload = json.dumps(
            _get_assembly_utterances(json.loads(self.file_object))
        )
        return self.payload

    def prepare_headers(self) -> Dict[str, str]:
        """Prepare headers for API request."""
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }
        return self.headers


@dataclass
class DeepgramSource(BaseSource):
    """
    Deepgram source object using a local or remote Deepgram JSON file.

    Parameters
    ----------
    filepath : Union[str, Path]
        The path to the local file.
    url : str
        The URL to the remote file.

    Raises
    ------
    ValueError
        If the file format is not valid.

    Examples
    --------
    >>> from wordcab.core_objects import DeepgramSource

    >>> deepgram_source = DeepgramSource(filepath="path/to/deepgram/file.json")  # doctest: +SKIP
    >>> deepgram_source  # doctest: +SKIP
    DeepgramSource(...)
    >>> deepgram_source.source  # doctest: +SKIP
    'deepgram'

    Returns
    -------
    DeepgramSource
        The Deepgram source object.
    """

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.source = "deepgram"

        if self._suffix != ".json":
            raise ValueError(
                f"Please provide a valid Deepgram file format. {self._suffix} is not valid, it should be .json."
            )

        if self.source_type == "local":
            self.file_object = self._load_file_from_path()
        elif self.source_type == "remote":
            self.file_object = self._load_file_from_url()

    def prepare_payload(self) -> str:
        """Prepare payload for API request."""
        self.payload = json.dumps(
            _get_deepgram_utterances(json.loads(self.file_object))
        )
        return self.payload

    def prepare_headers(self) -> Dict[str, str]:
        """Prepare headers for API request."""
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }
        return self.headers


@dataclass
class RevSource(BaseSource):
    """
    Rev.ai source object using a local or remote Rev.ai JSON file.

    Parameters
    ----------
    filepath : Union[str, Path]
        The path to the local file.
    url : str
        The URL to the remote file.

    Raises
    ------
    ValueError
        If the file format is not valid.

    Examples
    --------
    >>> from wordcab.core_objects import RevSource

    >>> rev_source = RevSource(filepath="path/to/rev/file.json")  # doctest: +SKIP
    >>> rev_source  # doctest: +SKIP
    RevSource(...)
    >>> rev_source.source  # doctest: +SKIP
    'rev_ai'

    Returns
    -------
    RevSource
        The Rev.ai source object.
    """

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.source = "rev_ai"

        if self._suffix != ".json":
            raise ValueError(
                f"Please provide a valid Rev.ai file format. {self._suffix} is not valid, it should be .json."
            )

        if self.source_type == "local":
            self.file_object = self._load_file_from_path()
        elif self.source_type == "remote":
            self.file_object = self._load_file_from_url()

    def prepare_payload(self) -> str:
        """Prepare payload for API request."""
        self.payload = json.dumps(_get_rev_monologues(json.loads(self.file_object)))
        return self.payload

    def prepare_headers(self) -> Dict[str, str]:
        """Prepare headers for API request."""
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
        }
        return self.headers


@dataclass
class VTTSource(BaseSource):
    """
    VTT source object using a local or remote VTT file.

    Parameters
    ----------
    filepath : Union[str, Path]
        The path to the local file.
    url : str
        The URL to the remote file.

    Raises
    ------
    ValueError
        If the file format is not valid.

    Examples
    --------
    >>> from wordcab.core_objects import VTTSource

    >>> vtt_source = VTTSource(filepath="path/to/vtt/file.vtt")  # doctest: +SKIP
    >>> vtt_source  # doctest: +SKIP
    VTTSource(...)
    >>> vtt_source.source  # doctest: +SKIP
    'vtt'

    Returns
    -------
    VTTSource
        The VTT source object.
    """

    filename: str = field(init=False)

    def __post_init__(self) -> None:
        """Post-init method."""
        super().__post_init__()
        self.source = "vtt"

        if self._suffix != ".vtt":
            raise ValueError(
                f"Please provide a valid VTT file format. {self._suffix} is not valid, it should be .vtt."
            )

        if self.source_type == "local":
            self.file_object = self._load_file_from_path()
        elif self.source_type == "remote":
            self.file_object = self._load_file_from_url()

        self.filename = self.filepath.name  # type: ignore

    def prepare_payload(self) -> bytes:
        """Prepare payload for API request."""
        self.payload: bytes = self.file_object
        return self.payload

    def prepare_headers(self) -> Dict[str, str]:
        """Prepare headers for API request."""
        self.headers = {
            "Content-Disposition": f"attachment; filename={self.filename}",
        }
        return self.headers



# Copyright 2022-2023 The Wordcab Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wordcab API Utils functions."""

from typing import Any, Dict, List, Optional, Union

from .config import (
    CONTEXT_ELEMENTS,
    EXTRACT_PIPELINES,
    SOURCE_LANG,
    SUMMARY_LENGTHS_RANGE,
    SUMMARY_PIPELINES,
    TARGET_LANG,
    TRANSCRIPT_SPEAKER_MAPPING,
)
from .core_objects.utils import _get_deepgram_utterances


def format_deepgram_source(deepgram_json: Dict[str, Any]) -> List[str]:
    """
    Format the Deepgram json object to a list of strings.

    Parameters
    ----------
    deepgram_json : Dict[str, Any]
        The Deepgram json object.

    Returns
    -------
    List[str]
        The formatted list of strings.
    """
    utterances = _get_deepgram_utterances(deepgram_json)

    final_utterances: List[str] = []
    utt_to_add: str = ""

    for utt in utterances:
        if not utt_to_add:
            utt_to_add = f"SPEAKER {TRANSCRIPT_SPEAKER_MAPPING[int(utt['speaker'])]}: {utt['transcript']}"
        else:
            if utt["speaker"] == utterances[utterances.index(utt) - 1]["speaker"]:
                utt_to_add = f"{utt_to_add} {utt['transcript']}"
            else:
                final_utterances.append(utt_to_add)
                utt_to_add = f"SPEAKER {TRANSCRIPT_SPEAKER_MAPPING[int(utt['speaker'])]}: {utt['transcript']}"

    if utt_to_add:
        final_utterances.append(utt_to_add)

    return final_utterances


def _check_context_elements(elements: Optional[Union[str, List[str]]]) -> bool:
    """
    Check the context elements.

    Parameters
    ----------
    elements : Optional[Union[str, List[str]]]
        The context elements.

    Returns
    -------
    bool
        True if the context elements are valid, False otherwise.
    """
    if elements is None:
        return True

    if isinstance(elements, str):
        elements = [elements]

    for element in elements:
        if element not in CONTEXT_ELEMENTS:
            return False

    return True


def _check_source_lang(lang: str) -> bool:
    """
    Check the source language.

    Parameters
    ----------
    lang : str
        The source language.

    Returns
    -------
    bool
        True if the source language is valid, False otherwise.
    """
    if lang not in SOURCE_LANG:
        return False

    return True


def _check_target_lang(lang: str) -> bool:
    """
    Check the target language.

    Parameters
    ----------
    lang : str
        The target language.

    Returns
    -------
    bool
        True if the target language is valid, False otherwise.
    """
    if lang not in TARGET_LANG:
        return False

    return True


def _check_summary_length(lengths: Union[int, List[int]]) -> bool:
    """
    Check the summary lengths.

    Parameters
    ----------
    lengths : Union[int, List[int]]
        The summary lengths.

    Returns
    -------
    bool
        True if the summary lengths are valid, False otherwise.
    """
    if isinstance(lengths, int):
        lengths = [lengths]

    for length in lengths:
        if length < SUMMARY_LENGTHS_RANGE[0] or length > SUMMARY_LENGTHS_RANGE[1]:
            return False

    return True


def _check_summary_pipelines(pipelines: Union[str, List[str]]) -> bool:
    """
    Check the summary pipelines.

    Parameters
    ----------
    pipelines : Union[str, List[str]]
        The summary pipelines.

    Returns
    -------
    bool
        True if the summary pipelines are valid, False otherwise.
    """
    if isinstance(pipelines, str):
        pipelines = [pipelines]

    for pipeline in pipelines:
        if pipeline not in SUMMARY_PIPELINES:
            return False

    return True


def _check_extract_pipelines(pipelines: Union[str, List[str]]) -> bool:
    """
    Check the extract pipelines.

    Parameters
    ----------
    pipelines : Union[str, List[str]]
        The extract pipelines.

    Returns
    -------
    bool
        True if the extract pipelines are valid, False otherwise.
    """
    if isinstance(pipelines, str):
        pipelines = [pipelines]

    for pipeline in pipelines:
        if pipeline not in EXTRACT_PIPELINES:
            return False

    return True


def _format_context_elements(elements: Union[str, List[str]]) -> str:
    """
    Format the context.

    Parameters
    ----------
    elements : Union[str, List[str]]
        The context elements.

    Returns
    -------
    str
        The formatted context.
    """
    if isinstance(elements, str):
        return elements

    return ",".join(elements)


def _format_lengths(lengths: Union[int, List[int]]) -> str:
    """
    Format the lengths.

    Parameters
    ----------
    lengths : Union[int, List[int]]
        The lengths.

    Returns
    -------
    str
        The formatted lengths.
    """
    if isinstance(lengths, int):
        return str(lengths)

    return ",".join([str(length) for length in lengths])


def _format_pipelines(pipelines: Union[str, List[str]]) -> str:
    """
    Format the pipelines.

    Parameters
    ----------
    pipelines : Union[str, List[str]]
        The pipelines.

    Returns
    -------
    str
        The formatted pipelines.
    """
    if isinstance(pipelines, str):
        return pipelines

    return ",".join(pipelines)


def _format_tags(tags: Union[str, List[str]]) -> str:
    """
    Format the tags.

    Parameters
    ----------
    tags : Union[str, List[str]]
        The tags.

    Returns
    -------
    str
        The formatted tags.
    """
    if isinstance(tags, str):
        return tags

    return ",".join(tags)
